'use strict';
var path = require('path');
var del = require('del');
var _ = require('lodash');
var glob = require('glob');
var zip = require('gulp-zip');
var chalk = require('chalk');


var extension_path = path.join(process.cwd(), 'extensions.json');
var extns = require(extension_path);

if (!extns.components.length && !extns.modules.length) {
  console.log(chalk.red('Joomla-watcher : extensions.json does not list any extensions to watch. Exiting...'));
  process.exit();
}

var joomlaDir = extns.joomlaDir;

if (glob.sync(path.join(extns.joomlaDir, 'configuration.php')).length !== 1) {
  console.log(chalk.red('Joomla-watcher : ' + chalk.blue.underline(extns.joomlaDir) + ' is not a valid joomla installation. Exiting...'));
  process.exit();
}

var defaults = [];
var release_zips = [];


module.exports = function(gulp) {

   gulp.task('clean:releases',function(){
      return del(['./releases/**'],{force:true});
   });
   release_zips.push('clean:releases');

  _.each(extns.components, function(comp) {

    var xml = glob.sync('**/' + comp + '.xml', {
      nodir: true,
      cwd: path.join(process.cwd(), 'components')
    });
    if (xml.length !== 1) {
      console.log(chalk.red(comp + '.xml' + ' not found for component ' + comp) + chalk.green('Exiting...'));
      process.exit();
    }

    var compDir = path.dirname(xml[0]);

    // clean joomla installation directory
    gulp.task('clean:' + comp + ':site', function() {
      return del([path.join(joomlaDir, 'components/com_' + comp)], {
        force: true
      })
    });
    gulp.task('clean:' + comp + ':admin', function() {
      return del([path.join(joomlaDir, 'administrator/components/com_' + comp)], {
        force: true
      })
    });
    gulp.task('clean:' + comp + ':sitelang', function() {
      return del([path.join(joomlaDir, 'language/**/*.com_' + comp + '.*')], {
        force: true
      })
    });
    gulp.task('clean:' + comp + ':adminlang', function() {
      return del([path.join(joomlaDir, 'administrator/language/**/*.com_' + comp + '.*')], {
        force: true
      })
    });

    // start copying development files to joomla installation
    gulp.task('copy:' + comp + ':xml', ['copy:' + comp + ':admin'], function() {
      return gulp.src('./components/' + compDir + '/*.xml')
        .pipe(gulp.dest(path.join(joomlaDir, 'administrator/components/com_' + comp + '/')));
    });

    gulp.task('copy:' + comp + ':site', ['clean:' + comp + ':site'], function() {
      return gulp.src('./components/' + compDir + '/site/{!(language)/**,!(language)}')
        .pipe(gulp.dest(path.join(joomlaDir, 'components/com_' + comp + '/')));
    });

    gulp.task('copy:' + comp + ':admin', ['clean:' + comp + ':admin'], function() {
      return gulp.src('./components/' + compDir + '/admin/{!(language)/**,!(language)}')
        .pipe(gulp.dest(path.join(joomlaDir, 'administrator/components/com_' + comp + '/')));
    });
    gulp.task('copy:' + comp + ':sitelang', ['clean:' + comp + ':sitelang'], function() {
      return gulp.src('./components/' + compDir + '/site/language/**')
        .pipe(gulp.dest(path.join(joomlaDir, '/language/')));
    });

    gulp.task('copy:' + comp + ':adminlang', ['clean:' + comp + ':adminlang'], function() {
      return gulp.src('./components/' + compDir + '/admin/language/**')
        .pipe(gulp.dest(path.join(joomlaDir, 'administrator/language/')));
    });

    gulp.task('zip:' + comp, function() {
      return gulp.src('./components/' + compDir + '/**')
        .pipe(zip(comp + '.zip'))
        .pipe(gulp.dest('releases'));
    });
    release_zips.push('zip:' + comp);

    var deps = ['copy:' + comp + ':site',
      'copy:' + comp + ':xml',
      'copy:' + comp + ':adminlang',
      'copy:' + comp + ':sitelang'
    ];

    gulp.task('watch:' + comp, function() {
      gulp.watch('./components/' + compDir + '/**', deps);
    });

    defaults = _.union(defaults, deps);

    defaults.push('watch:' + comp);

  });

  _.each(extns.modules, function(mod) {
    var xml = glob.sync('**/' + mod.name + '.xml', {
      nodir: true,
      cwd: path.join(process.cwd(), 'modules')
    });

    if (xml.length !== 1) {
      console.log(chalk.red(mod.name + '.xml' + ' not found for module ' + mod.name) +
        chalk.green(' Exiting...'));
      process.exit();
    }

    var modDir = path.dirname(xml[0]);

    if (mod.type !== 'site' && mod.type !== 'administrator') {
      console.log(chalk.red(mod.name + ' does not have a valid type.') + chalk.green(' Exiting...'));
      process.exit();
    }
    if (mod.type === 'site') {
      gulp.task('clean:' + mod.name, function() {
        return del([path.join(joomlaDir, 'modules/' + mod.name)], {
          force: true
        });
      });

      gulp.task('clean:' + mod.name + ':lang', function() {
        return del([path.join(joomlaDir, 'language/**/*.' + mod.name + '.*')], {
          force: true
        });
      });

      gulp.task('copy:' + mod.name + ':lang', ['clean:' + mod.name + ':lang'], function() {
        return gulp.src('./modules/' + modDir + '/language/**')
          .pipe(gulp.dest(path.join(joomlaDir, '/language/')));
      });

      gulp.task('copy:' + mod.name, ['clean:' + mod.name], function() {
        return gulp.src('./modules/' + modDir + '/{!(language)/**,!(language)}')
          .pipe(gulp.dest(path.join(joomlaDir, '/modules/' + mod.name + '/')));
      });
    }
    if (mod.type === 'administrator') {
      gulp.task('clean:' + mod.name, function() {
        return del([path.join(joomlaDir, 'administrator/modules/' + mod.name)], {
          force: true
        });
      });

      gulp.task('clean:' + mod.name + ':lang', function() {
        return del([path.join(joomlaDir, 'administrator/language/**/*.' + mod.name + '.*')], {
          force: true
        });
      });
      gulp.task('copy:' + mod.name + ':lang', ['clean:' + mod.name + ':lang'], function() {
        return gulp.src('./modules/' + modDir + '/language/**')
          .pipe(gulp.dest(path.join(joomlaDir, 'administrator/language/')));
      });
      gulp.task('copy:' + mod.name, ['clean:' + mod.name], function() {
        return gulp.src('./modules/' + modDir + '/{!(language)/**,!(language)}')
          .pipe(gulp.dest(path.join(joomlaDir, 'administrator/modules/' + mod.name + '/')));
      });
    }

    gulp.task('zip:' + mod.name, function() {
      return gulp.src('./modules/' + modDir + '/**')
        .pipe(zip(mod.name + '.zip'))
        .pipe(gulp.dest('./releases'));
    });
    release_zips.push('zip:' + mod.name);

    gulp.task('watch:' + mod.name, function() {
      gulp.watch('./modules/' + modDir + '/**', ['copy:' + mod.name,'copy:'+mod.name+':lang']);
    })

    defaults = _.union(defaults, ['copy:' + mod.name, 'watch:' + mod.name,'copy:'+mod.name+':lang']);

  });

  /* @TODO: Validate joomla xml file */

  gulp.task('release:packages', release_zips, function() {
    var packageFile = glob.sync('pkg_*.xml');
    if (packageFile.length !== 1) return console.log(chalk.blue('No package file found for creating single package file'));

    var packages = gulp.src(['./releases/**', './pkg_*.xml'])
      .pipe(zip(packageFile[0].split('.xml')[0] + '.zip'));

    return packages.pipe(gulp.dest('releases'));

  });

  return defaults;
}
